#include <util/delay.h>
#include "conf/config.h"
#include "conf/conf_usb.h"
#include "device_task.h"
#include "lib_mcu/usb/usb_drv.h"
#include "usb_descriptors.h"
#include "modules/usb/device_chap9/usb_standard_request.h"
#include "lib_mcu/wdt/wdt_drv.h"

extern   U8 jump_bootloader;
volatile U8 cpt_sof;
int query_status=0;
U8	usb_cmd;
U8	led_data;
U8	cds_status0;
U8	cds_status1;
U8	pws_good;
U8	pws_alert;

void device_task_init(void)
{
	int i;
	
	for(i=0;i<7;i++) {
		LedOn(i);
		_delay_ms(60);
		LedOff(i);
	}
	for(i=6;i>=0;i--) {
		LedOn(i);
		_delay_ms(60);
		LedOff(i);
	}
	LED_PORT = 0xfe;
	_delay_ms(200);
	LED_PORT = 0;	
}

void device_task(void)
{
	if(!Is_device_enumerated())          // Check USB HID is enumerated
	return;
	
	hid_report_out();
	hid_report_in();
}

void wait50us(unsigned int time){     // 100SPʂwait[` (NbNWMHẑƂ)
	unsigned char lpcnt;            // 6600SƂȂ i1MHzȂP800sj
	__asm__ __volatile__("\n"
	"CPU_wait_entry%=:\n\t"
	"ldi %0,200\n"          //CK 100us/8MHz
	"CPU_wait_lp%=:\n\t"
	"nop\n\t"
	"dec %0\n\t"
	"brne CPU_wait_lp%=\n\t"
	"sbiw %1,1\n\t"
	"brne CPU_wait_entry%=\n\t"
	:"=&a"(lpcnt)
	:"w"(time)
	);
	return;
}
void wait_cds_clock()
{
	wait50us(2);
	wait50us(2);
	wait50us(2);
}
void wait_cds_clock2()
{
	wait50us(2);
}
void get_cds_status(void)
{
	int i;
	U8	cds = 0;
	CDS_PORT	&= ~((1<<CDS_DS)|(1<<CDS_CLK));
	wait_cds_clock();
	CDS_PORT	|= (1<<CDS_DS);
	for( i = 0; i < 8; i++ ) {
		cds	= cds << 1;
		CDS_PORT	|= (1<<CDS_CLK);
		wait_cds_clock();
		CDS_PORT	&= ~(1<<CDS_CLK);
		if( (CDS_PIN&(1<<CDS_DATA)) != 0 ) {
			cds	|= 1;
		}
		wait_cds_clock2();
	}
	cds_status0	= cds;
	cds = 0;
	for( i = 0; i < 4; i++ ) {
		cds	= cds << 1;
		CDS_PORT	|= (1<<CDS_CLK);
		wait_cds_clock();
		CDS_PORT	&= ~(1<<CDS_CLK);
		if( (CDS_PIN&(1<<CDS_DATA)) != 0 ) {
			cds	|= 1;
		}
		wait_cds_clock2();
	}
	CDS_PORT	&= ~(1<<CDS_DS);
	cds_status1 = cds;
}
void get_power_state(void)
{
	pws_good	= (PW_PIN & (1<<PW_ALERT)) != 0 ? 1 : 0;
	pws_alert	= (PW_PIN & (1<<PW_DOWN)) != 0 ? 1 : 0;
}
void hid_report_out(void)
{
	U8 dummy;
	Usb_select_endpoint(EP_HID_OUT);
	if( Is_usb_receive_out() ) {
		usb_cmd = Usb_read_byte();
		if( usb_cmd == 'L' ) {
			dummy = Usb_read_byte();
			led_data = Usb_read_byte();
			dummy = Usb_read_byte();
			dummy = Usb_read_byte();
			dummy = Usb_read_byte();
			dummy = Usb_read_byte();
			dummy = Usb_read_byte();
			get_power_state();
			if( !pws_good || pws_alert ) {
				led_data = 0;
			}
			LED_PORT	= led_data;
			Usb_ack_receive_out();
			return;
		}
		Usb_ack_receive_out();
	}
}

void hid_report_in(void)
{
	Usb_select_endpoint(EP_HID_IN);
	if(!Is_usb_write_enabled())
		return;                                // Not ready to send report
	Usb_write_byte(cds_status0);	// CdS status
	Usb_write_byte(cds_status1);	// CdS status
	Usb_write_byte(pws_good);		// power alert
	Usb_write_byte(pws_alert);		// power down
	Usb_write_byte(PW_PIN);			// Dummy (not used)
	Usb_write_byte(0);				// Dummy (not used)
	Usb_write_byte(0);				// Dummy (not used)
	Usb_write_byte(0);				// Dummy (not used)
	Usb_ack_in_ready();				// Send data over the USB
	if( pws_good && !pws_alert ) {
		get_cds_status();
	}		
	get_power_state();
	return;
}

//! @brief sof_action
//!
//! This function increments the cpt_sof counter each times
//! the USB Start Of Frame interrupt subroutine is executed (1ms)
//! Usefull to manage time delays
//!
//! @param none
//!
//! @return none
void sof_action()
{
   cpt_sof++;
}
